@@ -34,10 +34,13 @@ Base_DB::~Base_DB()
 {
 }
 
-void Base_DB::init_db_details(const Glib::ustring& strDatabaseName)
+void Base_DB::init_db_details()
 {
-  m_strDatabaseName = strDatabaseName;
+  fill_from_database();
+}
 
+void Base_DB::refresh_db_details()
+{
   fill_from_database();
 }
 
@@ -54,7 +57,7 @@ void Base_DB::fill_end()
 //static:
 sharedptr<SharedConnection> Base_DB::connect_to_server()
 {
-  Bakery::BusyCursor(*get_app_window());
+  Bakery::BusyCursor(*get_application());
 
   sharedptr<SharedConnection> result(0);
 
@@ -67,15 +70,11 @@ sharedptr<SharedConnection> Base_DB::connect_to_server()
   return result;
 }
 
-
-Glib::ustring Base_DB::get_database_name()
-{
-  return m_strDatabaseName;
-}
-
 void Base_DB::handle_error(const std::exception& ex)
 {
-  Gtk::MessageDialog dialog(Glib::ustring("Internal error:\n") + ex.what(), Gtk::MESSAGE_WARNING );
+  Gtk::MessageDialog dialog(gettext("<b>Internal error</b>"), true, Gtk::MESSAGE_WARNING );
+  dialog.set_secondary_text(ex.what());
+  //TODO: dialog.set_transient_for(*get_application());
   dialog.run();
 }
 
@@ -101,7 +100,9 @@ bool Base_DB::handle_error()
         std::cerr << "Internal error: " << error_details << std::endl;
       }
 
-      Gtk::MessageDialog dialog(Glib::ustring("Internal error:\n") + error_details, Gtk::MESSAGE_WARNING );
+      Gtk::MessageDialog dialog(Glib::ustring("<b>Internal error</b>"), true, Gtk::MESSAGE_WARNING );
+      dialog.set_secondary_text(error_details);
+      //TODO: dialog.set_transient_for(*get_application());
       dialog.run();
 
       return true; //There really was an error.
@@ -117,7 +118,7 @@ Glib::RefPtr<Gnome::Gda::DataModel> Base_DB::Query_execute(const Glib::ustring&
 {
   Glib::RefPtr<Gnome::Gda::DataModel> result;
 
-  Bakery::BusyCursor(*get_app_window());
+  Bakery::BusyCursor(*get_application());
 
   sharedptr<SharedConnection> sharedconnection = connect_to_server();
   if(sharedconnection)
@@ -282,10 +283,14 @@ Base_DB::type_vecFields Base_DB::get_fields_for_table_from_database(const Glib::
     DATAMODEL_FIELDS_COL_DEFAULTVALUE = 8
   };
 
-  Bakery::BusyCursor(*get_app_window());
+  Bakery::BusyCursor(*get_application());
 
   sharedptr<SharedConnection> sharedconnection = connect_to_server();
-  if(sharedconnection)
+  if(!sharedconnection)
+  {
+    g_warning("Base_DB::get_fields_for_table_from_database(): connection failed.");
+  }
+  else  
   {
     Glib::RefPtr<Gnome::Gda::Connection> connection = sharedconnection->get_gda_connection();
 
@@ -294,11 +299,20 @@ Base_DB::type_vecFields Base_DB::get_fields_for_table_from_database(const Glib::
     param_list.add_parameter(param_table_name);
 
     Glib::RefPtr<Gnome::Gda::DataModel> data_model_fields = connection->get_schema(Gnome::Gda::CONNECTION_SCHEMA_FIELDS, param_list);
-    if(data_model_fields && (data_model_fields->get_n_columns() == 0))
+
+    if(!data_model_fields)
+    {
+      std::cerr << "Base_DB_Table_Definition::fill_fields(): libgda reported empty fields schema data_model for the table." << std::endl;
+    } 
+    else if(data_model_fields->get_n_columns() == 0)
     {
       std::cerr << "Base_DB_Table_Definition::fill_fields(): libgda reported 0 fields for the table." << std::endl;
     }
-    else if(data_model_fields)
+    else if(data_model_fields->get_n_rows() == 0)
+    {
+      g_warning("Base_DB::get_fields_for_table_from_database(): data_model_fields->get_n_rows() == 0: The table probably does not exist in the specified database.");
+    }
+    else
     {
       guint row = 0;
       guint rows_count = data_model_fields->get_n_rows();
@@ -368,6 +382,11 @@ Base_DB::type_vecFields Base_DB::get_fields_for_table_from_database(const Glib::
     }
   }
 
+  if(result.empty())
+  {
+    g_warning("Base_DB::get_fields_for_table_from_database(): returning empty result.");
+  }
+  
   return result;
 }
 
@@ -384,7 +403,7 @@ Base_DB::type_vecFields Base_DB::get_fields_for_table(const Glib::ustring& table
     type_vecFields result;
 
     type_vecFields fieldsDocument = pDoc->get_table_fields(table_name);
-
+  
     //Look at each field in the database:
     for(type_vecFields::iterator iter = fieldsDocument.begin(); iter != fieldsDocument.end(); ++iter)
     {
@@ -415,7 +434,7 @@ Base_DB::type_vecFields Base_DB::get_fields_for_table(const Glib::ustring& table
         result.push_back(*iter);
       }
     }
-
+  
     //Add any fields that are in the database, but not in the document:
     for(type_vecFields::iterator iter = fieldsDatabase.begin(); iter != fieldsDatabase.end(); ++iter)
     {
@@ -427,7 +446,6 @@ Base_DB::type_vecFields Base_DB::get_fields_for_table(const Glib::ustring& table
        //Add it if it is not there:
        if(iterFind == result.end() )
          result.push_back(*iter);
-
     }
 
     return result;
@@ -435,3 +453,42 @@ Base_DB::type_vecFields Base_DB::get_fields_for_table(const Glib::ustring& table
 
 }
 
+bool Base_DB::util_string_has_whitespace(const Glib::ustring& text)
+{
+ for(Glib::ustring::const_iterator iter = text.begin(); iter != text.end(); ++iter)
+ {
+   if(Glib::Unicode::isspace(*iter))
+     return true; //White space was found.
+ }
+
+ return false; //No white space found.
+}
+
+Glib::ustring Base_DB::util_title_from_string(const Glib::ustring& text)
+{
+  Glib::ustring result;
+
+  bool capitalise_next_char = true;
+  for(Glib::ustring::const_iterator iter = text.begin(); iter != text.end(); ++iter)
+  {
+    
+    const gunichar& ch = *iter;
+    if(ch == '_') //Replace _ with space.
+    {
+      capitalise_next_char = true; //Capitalise all words.
+      result += " ";
+    }
+    else
+    {
+      if(capitalise_next_char)
+        result += Glib::Unicode::toupper(*iter);
+      else
+        result += *iter;
+
+      capitalise_next_char = false;
+    }
+  }
+
+  return result;
+}
+
