@@ -27,7 +27,8 @@ LayoutItem_Field::LayoutItem_Field()
 LayoutItem_Field::LayoutItem_Field(const LayoutItem_Field& src)
 : LayoutItem(src),
   m_field(src.m_field),
-  m_table_name(src.m_table_name)
+  //m_table_name(src.m_table_name),
+  m_relationship_name(src.m_relationship_name)
 {
 }
 
@@ -46,11 +47,13 @@ LayoutItem_Field& LayoutItem_Field::operator=(const LayoutItem_Field& src)
   LayoutItem::operator=(src);
 
   m_field = src.m_field;
-  m_table_name = src.m_table_name;
+  //m_table_name = src.m_table_name;
+  m_relationship_name = src.m_relationship_name;
 
   return *this;
 }
 
+/*
 Glib::ustring LayoutItem_Field::get_table_name() const
 {
   return m_table_name;
@@ -60,5 +63,22 @@ void LayoutItem_Field::set_table_name(const Glib::ustring& table_name)
 {
   m_table_name = table_name;
 }
+*/
+
+bool LayoutItem_Field::get_has_relationship_name() const
+{
+  return !m_relationship_name.empty();
+}
+
+Glib::ustring LayoutItem_Field::get_relationship_name() const
+{
+  return m_relationship_name;
+}
+
+void LayoutItem_Field::set_relationship_name(const Glib::ustring& relationship_name)
+{
+  m_relationship_name = relationship_name;
+}
+
 
 
