@@ -54,6 +54,8 @@ public:
 
   virtual bool get_relationship(const Glib::ustring& table_name, const Glib::ustring& relationship_name, Relationship& relationship) const;
   
+  void set_relationship(const Glib::ustring& table_name, const Relationship& relationship);
+  
   typedef std::vector<Field> type_vecFields;
   virtual type_vecFields get_table_fields(const Glib::ustring& table_name) const;
   virtual void set_table_fields(const Glib::ustring& table_name, const type_vecFields& vecFields);
@@ -63,10 +65,14 @@ public:
 
   typedef std::map<guint, LayoutGroup> type_mapLayoutGroupSequence;
   virtual type_mapLayoutGroupSequence get_data_layout_groups(const Glib::ustring& layout_name, const Glib::ustring& table_name) const;
+  
   virtual void set_data_layout_groups(const Glib::ustring& layout_name, const Glib::ustring& table_name, const type_mapLayoutGroupSequence& groups);
 
+  virtual void set_relationship_data_layout_groups(const Glib::ustring& layout_name, const Relationship& relationship, const type_mapLayoutGroupSequence& groups);
+    
   virtual type_mapLayoutGroupSequence get_data_layout_groups_plus_new_fields(const Glib::ustring& layout_name, const Glib::ustring& table_name) const;
 
+  virtual type_mapLayoutGroupSequence get_relationship_data_layout_groups_plus_new_fields(const Glib::ustring& layout_name, const Relationship& relationship) const;
         
   ///When a field name is changed, change it in the relationships and fields data:
   virtual void change_field_name(const Glib::ustring& table_name, const Glib::ustring& strFieldNameOld, const Glib::ustring& strFieldNameNew);
@@ -89,12 +95,25 @@ public:
 
   virtual Glib::ustring get_database_title() const;
   virtual void set_database_title(const Glib::ustring& title);
+  
+  enum userLevelReason
+  {
+    USER_LEVEL_REASON_UNKNOWN,
+    USER_LEVEL_REASON_FILE_READ_ONLY,
+    USER_LEVEL_REASON_DATABASE_ACCESS_LEVEL
+  };
+  
+  /**
+   * @param reason The reason that the user is not a developer, if he is not.
+   * @result Whether the user is a developer.
+   */
+  virtual AppState::userlevels get_userlevel(userLevelReason& reason) const;
     
   virtual AppState::userlevels get_userlevel() const;
 
   /** This is transitory information, not saved to disk.
    */
-  virtual void set_userlevel(AppState::userlevels userlevel);
+  virtual bool set_userlevel(AppState::userlevels userlevel);
   
   typedef sigc::signal<void, AppState::userlevels> type_signal_userlevel_changed;
   type_signal_userlevel_changed signal_userlevel_changed();
@@ -107,7 +126,7 @@ protected:
   virtual bool load_after();
   virtual bool save_before();
 
-  virtual void load_after_layout_group(const xmlpp::Element* node, LayoutGroup& group);
+  virtual void load_after_layout_group(const xmlpp::Element* node, const Glib::ustring table_name, LayoutGroup& group);
   virtual void save_before_layout_group(xmlpp::Element* node, const LayoutGroup& group);
   
   virtual void on_app_state_userlevel_changed(AppState::userlevels userleve);
