@@ -40,13 +40,13 @@ Box_Data_List::Box_Data_List()
       m_pDialogLayout->signal_hide().connect( sigc::mem_fun(*this, &Box_Data::on_dialog_layout_hide) );
     }
   }
-  
+
   m_strHint = gettext("When you change the data in a field the database is updated immediately.\n Click [Add] or enter data into the last row to add a new record.\n Leave automatic ID fields empty - they will be filled for you.\nOnly the first 100 records are shown.");
 
   pack_start(m_AddDel);
   m_AddDel.set_auto_add(false); //We want to add the row ourselves when the user clicks the Add button, because the default behaviour there is not suitable.
   m_AddDel.set_rules_hint(); //Use alternating row colors when the theme does that.
-  
+
   //Connect signals:
   m_AddDel.signal_user_requested_add().connect(sigc::mem_fun(*this, &Box_Data_List::on_adddel_user_requested_add)); //Only emitted when m_AddDel.set_auto_add(false) is used.
   m_AddDel.signal_user_requested_edit().connect(sigc::mem_fun(*this, &Box_Data_List::on_adddel_user_requested_edit));
@@ -54,7 +54,7 @@ Box_Data_List::Box_Data_List()
   m_AddDel.signal_user_added().connect(sigc::mem_fun(*this, &Box_Data_List::on_adddel_user_added));
   m_AddDel.signal_user_changed().connect(sigc::mem_fun(*this, &Box_Data_List::on_adddel_user_changed));
   m_AddDel.signal_user_reordered_columns().connect(sigc::mem_fun(*this, &Box_Data_List::on_adddel_user_reordered_columns));
-  
+
   m_AddDel.signal_user_requested_layout().connect(sigc::mem_fun(*this, &Box_Data_List::on_adddel_user_requested_layout));
 
 
@@ -71,7 +71,7 @@ Box_Data_List::~Box_Data_List()
 void Box_Data_List::fill_from_database()
 {
   Bakery::BusyCursor(*get_app_window());
- 
+
   try
   {
     sharedptr<SharedConnection> sharedconnection = connect_to_server();
@@ -82,101 +82,88 @@ void Box_Data_List::fill_from_database()
 
     //Field Names:
     fill_column_titles();
-   
-    if(sharedconnection)
-    {    
-      Glib::RefPtr<Gnome::Gda::Connection> connection = sharedconnection->get_gda_connection();
 
-      Glib::ustring strWhereClausePiece;
-      if(!m_strWhereClause.empty())
-        strWhereClausePiece = " WHERE " + m_strWhereClause;
+    //if(sharedconnection)
+    //{
+      //Glib::RefPtr<Gnome::Gda::Connection> connection = sharedconnection->get_gda_connection();
 
-      m_Fields = get_fields_to_show();
+    Glib::ustring strWhereClausePiece;
+    if(!m_strWhereClause.empty())
+      strWhereClausePiece = " WHERE " + m_strWhereClause;
 
-      //Add extra possibly-non-visible columns that we need:
-      type_vecFields fieldsToGet = m_Fields;
-      fieldsToGet.push_back(m_AddDel.get_key_field());
-      const int index_primary_key = fieldsToGet.size() - 1;
+    m_Fields = get_fields_to_show();
 
-      if(!fieldsToGet.empty())
+    //Add extra possibly-non-visible columns that we need:
+    type_vecLayoutFields fieldsToGet = m_Fields;
+
+    LayoutItem_Field layout_item;
+    Field field_key = m_AddDel.get_key_field();
+    layout_item.set_name(field_key.get_name());
+    layout_item.m_field = field_key;
+    fieldsToGet.push_back(layout_item);
+    const int index_primary_key = fieldsToGet.size() - 1;
+
+    if(!fieldsToGet.empty())
+    {
+      const Glib::ustring query = build_sql_select_with_where_clause(m_strTableName, fieldsToGet, m_strWhereClause);
+
+      Glib::RefPtr<Gnome::Gda::DataModel> result = Query_execute(query);
+      if(!result)
+        handle_error();
+      else
       {
-        Glib::ustring sql_part_fields;
-        for(type_vecFields::const_iterator iter =  fieldsToGet.begin(); iter != fieldsToGet.end(); ++iter)
+        //Field contents:
+        m_AddDel.remove_all();
+
+        //each row:
+        guint rows_count = result->get_n_rows();
+        if(rows_count)
+          m_has_one_or_more_records = true; //Save it for later.
+
+        if(rows_count > 100)
+          rows_count = 100; //Don't get more than 100. TODO: Get other rows dynamically.
+
+        for(guint result_row = 0; result_row < (rows_count); result_row++)
         {
-          if(iter != fieldsToGet.begin())
-            sql_part_fields += ",";
-
-          sql_part_fields += iter->get_name();
-        }
-
-        //TODO: Remove hardcoded Limit 100.
-        std::stringstream query;
-        query << "SELECT " << sql_part_fields << " FROM " << m_strTableName << strWhereClausePiece; // << " LIMIT 100";
-        Glib::RefPtr<Gnome::Gda::DataModel> result = Query_execute(query.str());
-        if(!result)
-          handle_error();
-        else
-        {
-          //Field contents:
-          m_AddDel.remove_all();
-
-          //each row:
-          guint rows_count = result->get_n_rows();
-          if(rows_count)
-            m_has_one_or_more_records = true; //Save it for later.
-
-          if(rows_count > 100)
-            rows_count = 100; //Don't get more than 100. TODO: Get other rows dynamically.
-
-          for(guint result_row = 0; result_row < (rows_count); result_row++)
+          Gnome::Gda::Value key = result->get_value_at(index_primary_key, result_row);
+          //It is usually an integer.
+          if(GlomConversions::value_is_empty(key))
+            g_warning("Box_Data_List::fill_from_database(): primary key value is empty");
+          else
           {
-            Gnome::Gda::Value key = result->get_value_at(index_primary_key, result_row);
-            //It is usually an integer.
-            if(GlomConversions::value_is_empty(key))
-              g_warning("Box_Data_List::fill_from_database(): primary key value is empty");
-            else
+            Gnome::Gda::Value value_primary_key = result->get_value_at(index_primary_key, result_row);
+
+            Gtk::TreeModel::iterator tree_iter = m_AddDel.add_item(value_primary_key);
+
+            type_vecLayoutFields::const_iterator iterFields = fieldsToGet.begin();
+
+            //each field:
+            //We use cols_count -1 because we have an extra field for the primary_key.
+            guint cols_count = result->get_n_columns() - 1;
+            for(guint uiCol = 0; uiCol < cols_count; uiCol++)
             {
-              Gnome::Gda::Value value_primary_key = result->get_value_at(index_primary_key, result_row);
+              const Gnome::Gda::Value value = result->get_value_at(uiCol, result_row);
+              const LayoutItem_Field& layout_item = *iterFields;
 
-              Gtk::TreeModel::iterator tree_iter = m_AddDel.add_item(value_primary_key);
+              //g_warning("list fill: field_name=%s", iterFields->get_name().c_str());
+              //g_warning("  value_as_string=%s", value.to_string().c_str());
 
-              type_vecFields::const_iterator iterFields = fieldsToGet.begin();
+              m_AddDel.set_value(tree_iter, layout_item, value);
+               //g_warning("addedel size=%d", m_AddDel.get_count());
 
-              //each field:
-              //We use cols_count -1 because we have an extra field for the primary_key.
-              guint cols_count = result->get_n_columns() - 1;
-              for(guint uiCol = 0; uiCol < cols_count; uiCol++)
-              {
-                Gnome::Gda::Value value = result->get_value_at(uiCol, result_row);
-                guint index = 0;
-
-                //g_warning("list fill: field_name=%s", iterFields->get_name().c_str());
-                //g_warning("  value_as_string=%s", value.to_string().c_str());
-
-                //TODO_Performance: This searches fieldsToGet again each time:
-                const Glib::ustring field_name = iterFields->get_name();
-                bool test = get_field_column_index(field_name, index);
-                ++iterFields;
-
-                if(test)
-                {
-                  m_AddDel.set_value(tree_iter, index, value);
-                  //g_warning("addedel size=%d", m_AddDel.get_count());
-                }
-                else
-                  g_warning("  get_field_column_index failed: field name=%s", field_name.c_str());
-              }
+              ++iterFields;
             }
           }
         }
-      } //If !fieldsToGet.empty()
-    }
-   
+      }
+    } //If !fieldsToGet.empty()
+
+
     //Select first record:
     Glib::RefPtr<Gtk::TreeModel> refModel = m_AddDel.get_model();
     if(refModel)
       m_AddDel.select_item(refModel->children().begin());
- 
+
     fill_end();
 
   }
@@ -199,7 +186,7 @@ void Box_Data_List::on_adddel_user_requested_add()
     {
       index_field_to_edit = index_primary_key;
 
-      Field fieldPrimaryKey = m_Fields[index_primary_key];
+      Field fieldPrimaryKey = m_Fields[index_primary_key].m_field;
       if(fieldPrimaryKey.get_field_info().get_auto_increment())
       {
         //Start editing in the first cell that is not the primary key:
@@ -246,7 +233,9 @@ void Box_Data_List::on_adddel_user_requested_delete(const Gtk::TreeModel::iterat
   if(rowStart)
   {
     //Ask the user for confirmation:
-    Gtk::MessageDialog dialog(gettext("Are you sure that you would like to delete this record? The data in this record will then be permanently lost."), false, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_NONE);
+    Gtk::MessageDialog dialog(gettext("<b>Delete record</b>"), true, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_NONE);
+    dialog.set_secondary_text(gettext("Are you sure that you would like to delete this record? The data in this record will then be permanently lost."));
+    dialog.set_transient_for(*get_app_window());
     dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
     dialog.add_button(Gtk::Stock::DELETE, Gtk::RESPONSE_OK);
     
@@ -293,16 +282,14 @@ void Box_Data_List::on_adddel_user_added(const Gtk::TreeModel::iterator& row)
       m_AddDel.set_value_key(row, primary_key_value);
 
       //Show the primary key in the row, if the primary key is visible:
-      guint primary_key_model_col_index = 0;
-      bool test = get_field_column_index(field_primary_key.get_name(), primary_key_model_col_index);
-      if(test)
+      const Gnome::Gda::FieldAttributes fieldInfo = field_primary_key.get_field_info();
+      //If it's an auto-increment, then get the value and show it:
+      if(fieldInfo.get_auto_increment())
       {
-        const Gnome::Gda::FieldAttributes fieldInfo = field_primary_key.get_field_info();
-        //If it's an auto-increment, then get the value and show it:
-        if(fieldInfo.get_auto_increment())
-        {
-          m_AddDel.set_value(row, primary_key_model_col_index, primary_key_value);
-        }
+        LayoutItem_Field layout_item;
+        layout_item.set_name(field_primary_key.get_name());
+        layout_item.m_field = field_primary_key;
+        m_AddDel.set_value(row, layout_item, primary_key_value);
       }
 
       //Allow derived class to respond to record addition.
@@ -341,40 +328,84 @@ void Box_Data_List::on_adddel_user_reordered_columns()
 
     Document_Glom::type_mapLayoutGroupSequence mapGroups;
     mapGroups[1] = group;
-    
+
     pDoc->set_data_layout_groups("list", m_strTableName, mapGroups);  
   }
 }
 
 void Box_Data_List::on_adddel_user_changed(const Gtk::TreeModel::iterator& row, guint col)
-{ 
-  const Gnome::Gda::Value primary_key_value = get_primary_key_value(row);
-  if(!GlomConversions::value_is_empty(primary_key_value)) //If the record's primary key is filled in:
+{
+  const Gnome::Gda::Value parent_primary_key_value = get_primary_key_value(row);
+  if(!GlomConversions::value_is_empty(parent_primary_key_value)) //If the record's primary key is filled in:
   {
     //Just update the record:
     try
     {
-      Field field_primary_key = m_AddDel.get_key_field();
+      LayoutItem_Field layout_field = m_AddDel.get_column_field(col);
 
-      const Glib::ustring field_name = m_AddDel.get_column_field(col);
-      Field field;
-      bool test = get_fields_for_table_one_field(m_strTableName, field_name, field);
-      if(test)
-      {        
-        const Gnome::Gda::Value field_value = m_AddDel.get_value(row, col);
-            
-        Glib::ustring strQuery = "UPDATE " + m_strTableName;
-        strQuery += " SET " +  field.get_name() + " = " + field.sql(field_value);
+      Glib::ustring table_name = m_strTableName;
+      Field primary_key_field;
+      Gnome::Gda::Value primary_key_value;
 
-        strQuery += " WHERE " + field_primary_key.get_name() + " = " + field_primary_key.sql(primary_key_value);
-        Query_execute(strQuery);  //TODO: Handle errors
-
-        //Get-and-set values for lookup fields, if this field triggers those relationships:
-        do_lookups(row, field, field_value, field_primary_key, primary_key_value);
+      if(!layout_field.get_has_relationship_name())
+      {
+        table_name = m_strTableName;
+        primary_key_field = m_AddDel.get_key_field();
+        primary_key_value = parent_primary_key_value;
       }
       else
       {
-        g_warning("Box_Data_List::on_adddel_user_changed(): get_fields_for_table_one_field() failed: m_strTableName=%s, field_name=%s", m_strTableName.c_str(), field_name.c_str());
+        //If it's a related field then discover the actual table that it's in,
+        //plus how to identify the record in that table.
+        const Glib::ustring relationship_name = layout_field.get_relationship_name();
+
+        Document_Glom* document = dynamic_cast<Document_Glom*>(get_document());
+
+        Relationship relationship;
+        bool test = document->get_relationship(m_strTableName, relationship_name, relationship);
+        if(test)
+        {
+          table_name = relationship.get_to_table();
+          const Glib::ustring to_field_name = relationship.get_to_field();
+          //Get the key field in the other table (the table that we will change)
+          bool test = get_fields_for_table_one_field(table_name, to_field_name, primary_key_field); //TODO_Performance.
+          if(test)
+          {
+            //Get the value of the corresponding key in the current table (that identifies the record in the table that we will change)
+            LayoutItem_Field layout_item;
+            layout_item.set_name(relationship.get_from_field());
+
+            primary_key_value = get_entered_field_data(layout_item);
+            //TODO: If the primary_key is empty, or not yet valid, should we create a new record. It probably has to be anoption in the relationship.
+          }
+          else
+          {
+            g_warning("Box_Data_Details::on_flowtable_field_edited(): key not found for edited related field.");
+          }
+        }
+      }
+
+
+      const Gnome::Gda::Value field_value = m_AddDel.get_value(row, layout_field);
+      const Field& field = layout_field.m_field;
+      const Glib::ustring strFieldName = layout_field.get_name();
+
+      Glib::ustring strQuery = "UPDATE " + table_name;
+      strQuery += " SET " +  /* table_name + "." + postgres does not seem to like the table name here */ strFieldName + " = " + field.sql(field_value);
+      strQuery += " WHERE " + table_name + "." + primary_key_field.get_name() + " = " + primary_key_field.sql(primary_key_value);
+      bool bTest = Query_execute(strQuery);
+      if(!bTest)
+      {
+        //Update failed.
+        fill_from_database(); //Replace with correct values.
+      }
+      else
+      {
+        //Get-and-set values for lookup fields, if this field triggers those relationships:
+        do_lookups(row, layout_field, field_value, primary_key_field, primary_key_value);
+
+        //Update related fields, if this field is used in the relationship:
+        refresh_related_fields(row, layout_field, field_value, primary_key_field, primary_key_value);
       }
     }
     catch(const std::exception& ex)
@@ -399,17 +430,64 @@ void Box_Data_List::on_adddel_user_changed(const Gtk::TreeModel::iterator& row,
 
 }
 
-void Box_Data_List::do_lookups(const Gtk::TreeModel::iterator& row, const Field& field_changed, const Gnome::Gda::Value& field_value, const Field& primary_key, const Gnome::Gda::Value& primary_key_value)
+void Box_Data_List::refresh_related_fields(const Gtk::TreeModel::iterator& row, const LayoutItem_Field& field_changed, const Gnome::Gda::Value& /* field_value */, const Field& primary_key, const Gnome::Gda::Value& primary_key_value)
 {
+  if(field_changed.get_has_relationship_name())
+    return; //TODO: Handle these too.
+
+  //Get values for lookup fields, if this field triggers those relationships:
+  //TODO_performance: There is a LOT of iterating and copying here.
+  const Glib::ustring strFieldName = field_changed.get_name();
+  type_vecLayoutFields fieldsToGet = get_related_fields(strFieldName);
+
+  if(!fieldsToGet.empty())
+  {
+    const Glib::ustring query = build_sql_select(m_strTableName, fieldsToGet, primary_key, primary_key_value);
+
+    Glib::RefPtr<Gnome::Gda::DataModel> result = Query_execute(query);
+    if(!result)
+      handle_error();
+    else
+    {
+      //Field contents:
+      if(result->get_n_rows())
+      {
+        type_vecLayoutFields::const_iterator iterFields = fieldsToGet.begin();
+
+        guint cols_count = result->get_n_columns();
+        for(guint uiCol = 0; uiCol < cols_count; uiCol++)
+        {
+          const Gnome::Gda::Value value = result->get_value_at(uiCol, 0 /* row */);
+          const LayoutItem_Field& layout_item = *iterFields;
+
+          //g_warning("list fill: field_name=%s", iterFields->get_name().c_str());
+          //g_warning("  value_as_string=%s", value.to_string().c_str());
+
+          m_AddDel.set_value(row, layout_item, value);
+            //g_warning("addedel size=%d", m_AddDel.get_count());
+
+          ++iterFields;
+        }
+      }
+    }
+  }
+}
+
+void Box_Data_List::do_lookups(const Gtk::TreeModel::iterator& row, const LayoutItem_Field& field_changed, const Gnome::Gda::Value& field_value, const Field& primary_key, const Gnome::Gda::Value& primary_key_value)
+{
+   if(field_changed.get_has_relationship_name())
+    return; //TODO: Handle these too.
+
    //Get values for lookup fields, if this field triggers those relationships:
    //TODO_performance: There is a LOT of iterating and copying here.
    const Glib::ustring strFieldName = field_changed.get_name();
    type_list_lookups lookups = get_lookup_fields(strFieldName);
    for(type_list_lookups::const_iterator iter = lookups.begin(); iter != lookups.end(); ++iter)
    {
+     const LayoutItem_Field& layout_Item = iter->first;
+
      const Relationship relationship = iter->second;
-     const Field field_lookup = iter->first;
-     const Glib::ustring field_lookup_name = field_lookup.get_name();
+     const Field& field_lookup = layout_Item.m_field;
 
      Field field_source;
      bool test = get_fields_for_table_one_field(relationship.get_to_table(), field_lookup.get_lookup_field(), field_source);
@@ -418,16 +496,11 @@ void Box_Data_List::do_lookups(const Gtk::TreeModel::iterator& row, const Field&
        Gnome::Gda::Value value = get_lookup_value(iter->second /* relationship */,  field_source /* the field to look in to get the value */, field_value /* Value of to and from fields */);
 
        //Add it to the view:
-       guint column_index = 0;
-       bool test = get_field_column_index(field_lookup_name, column_index);
-       if(test)
-       {
-         m_AddDel.set_value(row, column_index, value);
-       }
+       m_AddDel.set_value(row, layout_Item, value);
 
        //Add it to the database (even if it is not shown in the view)
        Glib::ustring strQuery = "UPDATE " + m_strTableName;
-       strQuery += " SET " + field_lookup_name + " = " + field_lookup.sql(value);
+       strQuery += " SET " + field_lookup.get_name() + " = " + field_lookup.sql(value);
        strQuery += " WHERE " + primary_key.get_name() + " = " + primary_key.sql(primary_key_value);
        Query_execute(strQuery);  //TODO: Handle errors
 
@@ -529,24 +602,14 @@ Gnome::Gda::Value Box_Data_List::get_primary_key_value_selected()
   return Gnome::Gda::Value(m_AddDel.get_value_key_selected());
 }
 
-Gnome::Gda::Value Box_Data_List::get_entered_field_data(const Field& field) const
+Gnome::Gda::Value Box_Data_List::get_entered_field_data(const LayoutItem_Field& field) const
 {
-  //Get text from widget:
-
-  guint index = 0;
-  bool test = get_field_column_index(field.get_name(), index);
-  if(test)
-    return m_AddDel.get_value_selected(index);
-  else
-    return Gnome::Gda::Value(); //null.
+  return m_AddDel.get_value_selected(field);
 }
 
-void Box_Data_List::set_entered_field_data(const Field& field, const Gnome::Gda::Value& value)
+void Box_Data_List::set_entered_field_data(const LayoutItem_Field& field, const Gnome::Gda::Value& value)
 {
-  guint index = 0;
-  bool test = get_field_column_index(field.get_name(), index);
-  if(test)
-    return m_AddDel.set_value_selected(index, value);
+  return m_AddDel.set_value_selected(field, value);
 }
 
 guint Box_Data_List::get_records_count() const
@@ -569,10 +632,10 @@ void Box_Data_List::fill_column_titles()
         m_AddDel.set_key_field(field_primary_key);
 
 
-    type_vecFields listFieldsToShow = get_fields_to_show();
+    type_vecLayoutFields listFieldsToShow = get_fields_to_show();
 
     //Add a column for each table field:
-    for(type_vecFields::const_iterator iter =  listFieldsToShow.begin(); iter != listFieldsToShow.end(); ++iter)
+    for(type_vecLayoutFields::const_iterator iter =  listFieldsToShow.begin(); iter != listFieldsToShow.end(); ++iter)
     {
       m_AddDel.add_column(*iter);
     }
@@ -595,10 +658,10 @@ bool Box_Data_List::get_field_column_index(const Glib::ustring& field_name, guin
 {
   //Initialize output parameter:
   index = 0;
-  
+
   //Get the index of the field with this name:
   guint i = 0;
-  for(type_vecFields::const_iterator iter = m_Fields.begin(); iter != m_Fields.end(); ++iter)
+  for(type_vecLayoutFields::const_iterator iter = m_Fields.begin(); iter != m_Fields.end(); ++iter)
   {
     if(iter->get_name() == field_name)
     {
