@@ -44,13 +44,13 @@ Box_Data_List_Related::Box_Data_List_Related()
   m_Alignment.add(m_AddDel);
   m_AddDel.show();
   add(m_Frame);
-  
+
   m_layout_name = "list_related"; //TODO: We need a unique name when 2 portals use the same table.
-  
+
   //Delete the dialog from the base class, because we don't use it.
   if(m_pDialogLayout)
     delete m_pDialogLayout;
-        
+
   Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "window_data_layout_list_related");
   if(refXml)
   {
@@ -153,14 +153,12 @@ void Box_Data_List_Related::on_record_added(const Gnome::Gda::Value& primary_key
   Gtk::TreeModel::iterator iter = m_AddDel.get_row(primary_key_value);
   if(iter)
   {
-    guint iKey = 0;
     Gnome::Gda::Value key_value;
     //m_key_field is the field in this table that must match another field in the parent table.
-    bool primary_key_is_visible = get_field_column_index(m_key_field.get_name(), iKey);
-    if(primary_key_is_visible)
-    {
-      key_value = m_AddDel.get_value(iter, iKey);
-    }
+    LayoutItem_Field layout_item;
+    layout_item.set_name(m_key_field.get_name());
+    layout_item.m_field = m_key_field;
+    key_value = m_AddDel.get_value(iter, layout_item);
 
     Box_Data_List::on_record_added(key_value); //adds blank row.
 
@@ -185,10 +183,14 @@ void Box_Data_List_Related::on_record_added(const Gnome::Gda::Value& primary_key
       strQuery += " SET " +  /* get_table_name() + "." +*/ m_key_field.get_name() + " = " + m_key_field.sql(m_key_value);
       strQuery += " WHERE " + get_table_name() + "." + field_primary_key.get_name() + " = " + field_primary_key.sql(primary_key_value);
       bool test = Query_execute(strQuery);
-      if(test && primary_key_is_visible)
+      if(test)
       {
         //Show it on the view, if it's visible:
-        m_AddDel.set_value(iter, iKey, m_key_value);
+        LayoutItem_Field layout_item;
+        layout_item.set_name( field_primary_key.get_name() );
+        layout_item.m_field = field_primary_key;
+
+        m_AddDel.set_value(iter, layout_item, m_key_value);
       }
 
       //on_adddel_user_changed(iter, iKey); //Update the database.
@@ -228,8 +230,9 @@ void Box_Data_List_Related::on_adddel_user_added(const Gtk::TreeModel::iterator&
   else
   {
     //Tell user that they can't do that:
-    Gtk::MessageDialog dialog(gettext("You attempted to add a new related record, \nbut there can only be one related record, \nbecause the relationship uses a unique key."),
-      Gtk::MESSAGE_WARNING);
+    Gtk::MessageDialog dialog(gettext("<b>Extra related records not possible</b>"), true, Gtk::MESSAGE_WARNING);
+    dialog.set_secondary_text(gettext("You attempted to add a new related record, but there can only be one related record, because the relationship uses a unique key.")),
+    dialog.set_transient_for(*get_app_window());
     dialog.run();
 
     //Replace with correct values:
@@ -241,50 +244,17 @@ void Box_Data_List_Related::enable_buttons()
 {
 }
 
-Box_Data_List_Related::type_vecFields Box_Data_List_Related::get_fields_to_show() const
+Box_Data_List_Related::type_vecLayoutFields Box_Data_List_Related::get_fields_to_show() const
 {
-  type_vecFields result;
-
   const Document_Glom* document = get_document();
   if(document)
   {
-
     Document_Glom::type_mapLayoutGroupSequence mapGroups = document->get_relationship_data_layout_groups_plus_new_fields(m_layout_name, m_relationship);
 
-    if(mapGroups.empty())
-    {
-      //Start with a suitable default set of fields:
-      result = get_table_fields_to_show(m_relationship.get_to_table());
-    }
-    else
-    {
-      type_vecFields all_db_fields = get_fields_for_table(m_relationship.get_to_table());
-
-      for(Document_Glom::type_mapLayoutGroupSequence::const_iterator iter = mapGroups.begin(); iter != mapGroups.end(); ++iter)
-      {
-        const LayoutGroup& group = iter->second;
-    
-        //Add the group's fields:
-        LayoutGroup::type_map_const_items items = group.get_items();
-        for(LayoutGroup::type_map_const_items::const_iterator iter = items.begin(); iter != items.end(); ++iter)
-        {
-          const LayoutItem_Field* item = dynamic_cast<const LayoutItem_Field*>(iter->second);
-          if(item)
-          {
-            type_vecFields::const_iterator iterFind = std::find_if(all_db_fields.begin(), all_db_fields.end(), predicate_FieldHasName<Field>(item->get_name())); //item->m_field->get_name() is not filled in yet.
-  
-            //If the field does not exist anymore then we won't try to show it:
-            if(iterFind != all_db_fields.end() )
-            {
-              result.push_back(*iterFind);
-            }
-          }
-        }
-      }
-    }
+    return get_table_fields_to_show(m_relationship.get_to_table(), mapGroups);
   }
 
-  return result;
+  return type_vecLayoutFields();
 }
 
 void Box_Data_List_Related::show_layout_dialog()
