@@ -34,8 +34,11 @@ Dialog_Layout_Details::Dialog_Layout_Details(BaseObjectType* cobject, const Glib
   m_button_field_add_group(0),
   m_button_add_related(0),
   m_button_field_delete(0),
-  m_button_field_edit(0)
+  m_button_field_edit(0),
+  m_label_table_name(0)
 {
+  refGlade->get_widget("label_table_name", m_label_table_name);
+
   refGlade->get_widget("treeview_fields", m_treeview_fields);
   if(m_treeview_fields)
   {
@@ -140,6 +143,7 @@ void Dialog_Layout_Details::fill_group(const Gtk::TreeModel::iterator& iter, Lay
     for(Gtk::TreeModel::iterator iterChild = row.children().begin(); iterChild != row.children().end(); ++iterChild)
     {
       Gtk::TreeModel::Row rowChild = *iterChild;
+
       if(rowChild[m_model_items->m_columns.m_col_type] == TreeStore_Layout::TYPE_GROUP)
       {
         //Recurse:
@@ -158,6 +162,9 @@ void Dialog_Layout_Details::fill_group(const Gtk::TreeModel::iterator& iter, Lay
         //Add field:
         LayoutItem_Field field;
         field.set_name( rowChild[m_model_items->m_columns.m_col_name] );
+        field.set_relationship_name( rowChild[m_model_items->m_columns.m_col_relationship_name] );
+
+        field.set_editable( rowChild[m_model_items->m_columns.m_col_editable] );
 
         group.add_item(field);
       }
@@ -216,6 +223,9 @@ void Dialog_Layout_Details::add_group(const Gtk::TreeModel::iterator& parent, co
             Gtk::TreeModel::Row row = *iterField;
             row[m_model_items->m_columns.m_col_type] = TreeStore_Layout::TYPE_FIELD;
             row[m_model_items->m_columns.m_col_name] = field->get_name();
+            row[m_model_items->m_columns.m_col_relationship_name] = field->get_relationship_name();
+
+            row[m_model_items->m_columns.m_col_editable] = field->get_editable();
           }
         }
       }
@@ -225,7 +235,7 @@ void Dialog_Layout_Details::add_group(const Gtk::TreeModel::iterator& parent, co
   }
 }
 
-void Dialog_Layout_Details::set_document(const Glib::ustring& layout, Document_Glom* document, const Glib::ustring& table_name, const type_vecFields& table_fields)
+void Dialog_Layout_Details::set_document(const Glib::ustring& layout, Document_Glom* document, const Glib::ustring& table_name, const type_vecLayoutFields& table_fields)
 {
   m_modified = false;
 
@@ -250,10 +260,9 @@ void Dialog_Layout_Details::set_document(const Glib::ustring& layout, Document_G
       group.m_columns_count = 2;
 
       guint field_sequence = 1; //0 means no sequence
-      for(type_vecFields::const_iterator iter = table_fields.begin(); iter != table_fields.end(); ++iter)
+      for(type_vecLayoutFields::const_iterator iter = table_fields.begin(); iter != table_fields.end(); ++iter)
       {
-        LayoutItem_Field item;
-        item.set_name(iter->get_name());
+        LayoutItem_Field item = *iter;
         item.m_sequence = field_sequence;
 
         group.add_item(item, field_sequence);
@@ -411,8 +420,8 @@ void Dialog_Layout_Details::on_button_field_add()
 {
   Gtk::TreeModel::iterator parent = get_selected_group_parent();
 
-  Field field;
-  bool test = offer_field_list(field);
+  LayoutItem_Field layout_item;
+  bool test = offer_field_list(layout_item);
   if(test)
   {
     //Add the field details to the layout treeview:
@@ -435,7 +444,8 @@ void Dialog_Layout_Details::on_button_field_add()
     {
       Gtk::TreeModel::Row row = *iter;
       row[m_model_items->m_columns.m_col_type] = TreeStore_Layout::TYPE_FIELD;
-      row[m_model_items->m_columns.m_col_name] = field.get_name();
+      row[m_model_items->m_columns.m_col_name] = layout_item.get_name();
+      row[m_model_items->m_columns.m_col_relationship_name] = layout_item.get_relationship_name();
       //row[m_model_items->m_columns.m_col_title] = field.get_title();
 
       //Scroll to, and select, the new row:
@@ -463,7 +473,9 @@ bool Dialog_Layout_Details::offer_relationship_list(Relationship& relationship)
     if(dialog)
     {
       dialog->set_document(m_document, m_table_name);
+      dialog->set_transient_for(*this);
       int response = dialog->run();
+      dialog->hide();
       if(response == Gtk::RESPONSE_OK)
       {
         //Get the chosen relationship:
@@ -481,10 +493,10 @@ bool Dialog_Layout_Details::offer_relationship_list(Relationship& relationship)
   return result;
 }
 
-bool Dialog_Layout_Details::offer_field_list(Field& field)
+bool Dialog_Layout_Details::offer_field_list(LayoutItem_Field& field)
 {
   bool result = false;
-  
+
   try
   {
     Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "dialog_choose_field");
@@ -494,7 +506,8 @@ bool Dialog_Layout_Details::offer_field_list(Field& field)
 
     if(dialog)
     {
-      dialog->set_document(m_document, m_table_name);
+      dialog->set_document(m_document, m_table_name, field);
+      dialog->set_transient_for(*this);
       int response = dialog->run();
       if(response == Gtk::RESPONSE_OK)
       {
@@ -638,14 +651,19 @@ void Dialog_Layout_Details::on_button_field_edit()
         }
         case TreeStore_Layout::TYPE_FIELD:
         {
-          Field field;
+          LayoutItem_Field field;
           field.set_name( row[m_model_items->m_columns.m_col_name] ); //Start with this one selected.
+          field.set_relationship_name( row[m_model_items->m_columns.m_col_relationship_name] );
+          field.set_editable( row[m_model_items->m_columns.m_col_editable] );
           bool test = offer_field_list(field);
           if(test)
           {
             row[m_model_items->m_columns.m_col_name] = field.get_name();
+            row[m_model_items->m_columns.m_col_relationship_name] = field.get_relationship_name();
+
+            row[m_model_items->m_columns.m_col_editable] = field.get_editable();
           }
-          
+
           break;
         }
         case TreeStore_Layout::TYPE_PORTAL:
@@ -657,7 +675,7 @@ void Dialog_Layout_Details::on_button_field_edit()
           {
             row[m_model_items->m_columns.m_col_relationship] = relationship.get_name();
           }
-          
+
           break;
         }
       }
@@ -717,7 +735,7 @@ void Dialog_Layout_Details::on_treeview_fields_selection_changed()
 void Dialog_Layout_Details::on_cell_data_name(Gtk::CellRenderer* renderer, const Gtk::TreeModel::iterator& iter)
 {
   //TODO: If we ever use this a real layout tree, then let's add icons for each type.
-  
+
   //Set the view's cell properties depending on the model's data:
   Gtk::CellRendererText* renderer_text = dynamic_cast<Gtk::CellRendererText*>(renderer);
   if(renderer_text)
@@ -741,12 +759,24 @@ void Dialog_Layout_Details::on_cell_data_name(Gtk::CellRenderer* renderer, const
         markup = gettext("Related: ") + row[m_model_items->m_columns.m_col_relationship];
       }
       else
-        markup = row[m_model_items->m_columns.m_col_name];
+      {
+        //It's a field:
+
+        //Indicate that it's a field in another table.
+        const Glib::ustring relationship = row[m_model_items->m_columns.m_col_relationship_name];
+        if(!relationship.empty())
+          markup = relationship + "::";
+
+        markup += row[m_model_items->m_columns.m_col_name];
+
+        //Just for debugging:
+       // if(!row[m_model_items->m_columns.m_col_editable])
+       //  markup += " *";
+      }
 
       renderer_text->property_markup() = markup;
 
-      renderer_text->property_editable() = row[m_model_items->m_columns.m_col_type] != TreeStore_Layout::TYPE_PORTAL; //Portals can only be edited via the Edit button.
- 
+      renderer_text->property_editable() = false; //Names can never be edited.
     }
   }
 }
