@@ -24,21 +24,28 @@
 #include "../application.h"
 #include "../mode_data/dialog_choose_field.h"
 
-
-
+/*
 DataWidget::DataWidget(Field::glom_field_type glom_type, const Glib::ustring& title)
 : m_glom_type(glom_type),
   m_pMenuPopup(0)
+*/
+
+DataWidget::DataWidget(const LayoutItem_Field& field, const Glib::ustring& table_name)
+: m_pMenuPopup(0)
 {
+  Field::glom_field_type glom_type = field.m_field.get_glom_type();
+  set_layout_item(field.clone(), table_name); //takes ownership
+
   Gtk::Widget* child = 0;
+  const Glib::ustring title = field.m_field.get_title_or_name();
   if(glom_type == Field::TYPE_BOOLEAN)
   {
     Gtk::CheckButton* checkbutton = Gtk::manage( new Gtk::CheckButton( title ) );
     checkbutton->show();
-    checkbutton->signal_toggled().connect( sigc::mem_fun(*this, &DataWidget::on_widget_edited)  )
+    checkbutton->signal_toggled().connect( sigc::mem_fun(*this, &DataWidget::on_widget_edited)  );
 
     //TODO: entry->signal_user_requested_layout().connect( sigc::mem_fun(*this, &DataWidget::on_child_user_requested_layout );
-;
+
     child = checkbutton;
 
     m_label.set_text( Glib::ustring() ); //It is not used.
@@ -283,7 +290,7 @@ bool DataWidget::on_button_press_event(GdkEventButton *event)
   return Gtk::EventBox::on_button_press_event(event);
 }
 
-bool DataWidget::offer_field_list(const Glib::ustring& table_name, Field& field)
+bool DataWidget::offer_field_list(const Glib::ustring& table_name, LayoutItem_Field& field)
 {
   bool result = false;
 
@@ -297,6 +304,7 @@ bool DataWidget::offer_field_list(const Glib::ustring& table_name, Field& field)
     if(dialog)
     {
       dialog->set_document(get_document(), table_name, field);
+      dialog->set_transient_for(*get_application());
       int response = dialog->run();
       dialog->hide();
       if(response == Gtk::RESPONSE_OK)
@@ -323,12 +331,9 @@ void DataWidget::on_menupopup_activate_layout()
   LayoutItem_Field* layoutField = dynamic_cast<LayoutItem_Field*>(get_layout_item());
   if(layoutField)
   {
-    Field field;
-    field.set_name(layoutField->get_name()); //So that the current field will be selected.
-    bool test = offer_field_list(layoutField->get_table_name(), field);
+    bool test = offer_field_list(m_table_name, *layoutField);
     if(test)
     {
-      layoutField->set_name(field.get_name());
       signal_layout_changed().emit();
     }
   }
